#### DATA

# Loading gene MA results
input_file <- "/clinicfs/userhomes/jfcatala/Proyectos/sclerosis/analysis/02b_Metaanalisis_genes/MA_20210809_nervous_Men/result_meta.RData"
load(input_file)
genes <- result_meta



# Setting paths
dir <- "/clinicfs/userhomes/jfcatala/Proyectos/sclerosis/analysis/05_EF_Metaanalisis_ED/"
out_file <- "EF_20210809_MA_20210809_nervous_Men.RData"


#### FUNCTIONAL ENRICHMENT
library(org.Hs.eg.db)
library(limma)
library(Biobase)
library(mdgsa)


# Functions. The functions are prepared for limma
annotat <- function(genes, func_term, propagate=TRUE, minblock=10, maxblock=500) {
  # GO terms
  if (func_term %in% c("bp","cc","mf","GO")) {
    anot2 <- select(org.Hs.eg.db, 
                    keys = unique(rownames(genes)), 
                    columns = c("SYMBOL", "GO"), 
                    keytype = "SYMBOL")
    anot2 <- anot2[!is.na(anot2[,"GO"]),]
    
    # Converts an annotation matrix with two columns (the first with the gene 
    # ids and the second with the annotation ids) into an annotation list. 
    # Each GO is associated with id_ensembl.
    anot <- annotMat2list(anot2)
    
    ## Split Ontologies
    if (func_term == "bp") {
      anot <- splitOntologies (anot, na.rm = TRUE, verbose = TRUE)
      anot <- anot$bp
    }
    if (func_term == "mf") {
      anot <- splitOntologies (anot, na.rm = TRUE, verbose = TRUE)
      anot <- anot$mf
    }
    if (func_term == "cc") {
      anot <- splitOntologies (anot, na.rm = TRUE, verbose = TRUE)
      anot <- anot$cc
    } 

    
    if (propagate == TRUE) { # Ppropagate annotation 
      anot <- propagateGO (anot, verbose = TRUE)
    }
    
    anot <- annotFilter(annot = anot,  
                        minBlockSize = minblock, 
                        maxBlockSize = maxblock)
    return(anot)
  }
  
  # KEGG pathways
  if (func_term == "KEGG") {
    anot2 <- select(org.Hs.eg.db, 
                    keys = unique(rownames(genes)), 
                    columns = c("SYMBOL", "PATH"), 
                    keytype = "SYMBOL")
    anot2 <- anot2[!is.na(anot2[,"PATH"]),]
    
    anot <- annotMat2list(anot2)
    
    return(anot)
  }
  
  # Reactome pathways
  if (func_term == "Reactome") {
    load("/clinicfs/userhomes/jfcatala/Proyectos/sclerosis/analysis/03_Enriquecimiento_funcional/reactome/human_symbol_reactome_biomart_20210304.RData")
    anot2 <- h_reac
    anot2 <- anot2[!is.na(anot2[,"reactome"]),]
    
    # Convierte una matriz de anotacion con dos columnas (la primera con los 
    # id de los genes y la segunda con los id de la anotacion) en una lista de 
    # anotacion. Cada GO con los id_ensembl asociados.
    anot <- annotMat2list(anot2)
    
    return(anot)
  }
  
  
  if (!func_term %in% c("bp","cc","mf","GO","KEGG","Reactome")) {
    stop("Bad functional term name")
  }
}

GSEA <- function(ED, annotat) {
  pvalue <- ED$pvalue
  names(pvalue) <- rownames(ED)
  stat <- ED$logFC
  names(stat) <- rownames(ED)
  
  # Checking
  print(table(names(pvalue) == names(stat)))
  
  ##RANKING INDEX
  # Transforms p.value and its associated value of t into a ranking.
  rindex0 <- pval2index (pval = pvalue, sign = stat)
  
  # Checking
  print(table(names(pvalue) == names(rindex0)))
  
  ##TRANSFORM
  # Transform the ranking to make its distribution suitable as an independent 
  # variable in a logistic regression model.
  rindex <- indexTransform (index = rindex0, method = "normalize")
  
  #uvGSA
  return(uvGsa(index = rindex, annot = annotat, fulltable = TRUE))
}

# Annotation
anotaciones <- list("bp","KEGG","Reactome")
names(anotaciones) <- c("bp","KEGG","Reactome")

annot <- lapply(anotaciones, function(x) {annotat(genes = genes, func_term = x)})

# GSA
enrich <- lapply(annot, function(x) {GSEA(genes, annotat = x)})


save.image(file = paste0(dir,"/",out_file))


#### Report
library(stringr)

report_name <- str_split(input_file, pattern = "/", simplify = T)
report_name <- report_name[,length(report_name)-1]


# Creating links to term names for the report
rn <- function(x, annot){
  if(annot == "bp"){
    library(mdgsa)
    x$name <- mdgsa::getGOnames(rownames(x))
    rownames(x) <- str_c("<a href=https://www.ebi.ac.uk/QuickGO/term/",rownames(x),
                       " target=_blank>",rownames(x)," </a>")
    
  }
  if(annot == "KEGG"){
    library(mdgsa)
    x$name <- mdgsa::getKEGGnames(str_remove(rownames(x), "hsa"))
    rownames(x) <- str_c("<a href=https://www.genome.jp/dbget-bin/www_bget?hsa",rownames(x),
                         " target=_blank> hsa",rownames(x)," </a>")
  }
  if(annot == "Reactome"){
    library(reactome.db)
    reactome_df <- as.data.frame(reactomePATHNAME2ID)
    reactome_df$path_name <- str_remove(reactome_df$path_name, ".+: ")
    x$name <- reactome_df$path_name[match(rownames(x),reactome_df$DB_ID)]
    rownames(x) <- str_c("<a href=https://reactome.org/content/detail/",rownames(x),
                         " target=_blank>",rownames(x)," </a>")
  }
  
  return(x)
}


for(name in names(enrich)){
  enrich[[name]] <- rn(enrich[[name]], name)
}

rownames(genes) <- str_c("<a href=https://www.genecards.org/cgi-bin/carddisp.pl?gene=",rownames(genes),
                         " target=_blank>",rownames(genes)," </a>")

genes$gene <- NULL

# Temp file
save.image(file = paste0(dir,"/",report_name,".RData"))

# Creating report tabs
library(stringr)
make.tabs <- function(annot){
  res <- NULL
  for(g in names(annot)){
      name <- toupper(g)
      res <- c(res, '## ', name, ' {.tabset .tabset-pills -} \n')
        res <- c(res, 
                 "```{r, fig.align='center'}", '\n',
                 "dt <- ",deparse(substitute(annot)),"[['",g,"']] \n",
                 "dt_sig <- dt[dt$padj < 0.05,] \n",
                 "caption <- paste0('Statistics of ', nrow(dt_sig), ' significant terms') \n",
                 "dt_sig[,c(1:6)] <- dt_sig[,c(1:6)] %>% mutate_if(is.numeric, ~round(., 3)) \n",
                 "DT::datatable(dt_sig[,c(8,2:6,1)], 
                 caption=caption, 
                 colnames=c('Name','LOR','p.valor','adj.p.valor BY','SD','t','N'), 
                 escape = FALSE, filter = 'top') \n",
                 '```', '\n\n')
      }
  return(res)
  }


# Generating Rmd code
cat(
'---
title: "Functional enrichment from MA results" \n',
'subtitle: From ', report_name,
' \n',
'output:
  html_document:
    toc: false
    toc_float: false
    code_folding: hide
    number_sections: true
    theme: spacelab
---

#  {.tabset .tabset-pills -}

```{r, warning=F, message=F}
library(DT)
library(stringr)
library(dplyr)
load("', dir, '/', report_name, '.RData")
``` \n

\n\n',

make.tabs(enrich), 
  
  "\n\n",

'## MA results  {-} \n',
"```{r, fig.align='center'}", '\n',
"caption <- paste0('Statistics of ', sum(genes$p.adjust.fdr < 0.05), ' significant genes') \n",
"DT::datatable(genes[genes$p.adjust.fdr < 0.05, c('logFC', 'pvalue', 'p.adjust.fdr', 'SE', 'n.studies')], 
caption=caption, colnames=c('logFC','pvalue','FDR','SE','n.studies'), 
escape = FALSE, filter = 'top') \n",
'```', '\n\n',  
'## sessionInfo {-}  \n',
  "```{r, fig.align='center'}", '\n',
  "date()", "\n",
  "input_file", '\n',
  "sessionInfo()", '\n',
  '```', '\n\n',
  sep = "",
  file = paste0(dir, "/", report_name, ".Rmd"))

# Compile the temporary file and delete it
rmarkdown::render(paste0(dir, "/", report_name, ".Rmd"))
file.remove(paste0(dir,"/",report_name,".RData"))
